#ifndef BatchLab_Array_h
#define BatchLab_Array_h

// Inheritance :
#include <BatchLab/Core/BaseObject.h>
#include <Slash/Data/IArray.h>
#include <Slash/Store/IStorable.h>

#include <BatchLab/Core/Annotation.h>
#include <Lib/Array.h>

namespace BatchLab {

class Array 
:public BaseObject
,public virtual Slash::Data::IArray 
,public virtual Slash::Store::IStorable
{
public: //AIDA/IManagedObject
  virtual void* cast(const std::string&) const;
  virtual std::string type() const; //AIDA-v3r3p0
public: //BatchLab/Slash::Data::IArray
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual unsigned int dimension() const;
  virtual unsigned int size() const;
  virtual const std::vector<unsigned int>& orders() const;
  virtual bool setValue(const std::vector<unsigned int>&,double);
  virtual bool value(const std::vector<unsigned int>&,double&) const;
  virtual bool fill(const std::vector<double>&,Cut* = 0);
  virtual bool copy(const Slash::Data::IArray& aFrom);
  virtual bool add(const Slash::Data::IArray& aArray,Cut* = 0);
  virtual bool subtract(const Slash::Data::IArray& aArray);
  virtual bool multiply(const Slash::Data::IArray& aArray);
  virtual bool divide(const Slash::Data::IArray& aArray);
  virtual bool add(double,Cut* = 0);
  virtual bool multiply(double);
  virtual bool invert();
  virtual bool indices(unsigned int,std::vector<unsigned int>&) const;
  virtual const std::vector<double>& vector() const;
  virtual std::vector<double>& vector();
  virtual bool accept(unsigned int,const Cut&) const;
public: //Slash::Store::IObject
  virtual Slash::Store::IObject* copy() const;
  virtual std::string storeClassName() const;
public: //IVisited
  virtual bool visit(Slash::Store::IConstVisitor&) const;
  virtual bool read(Slash::Store::IVisitor&);
public:
  Array(Slash::Core::IWriter&,
         const std::string&,
         const std::string&,
         const std::vector<unsigned int>&);
  Array(const Array&);
  virtual ~Array();
  SLASH_STORE_HEADER
private:
  Annotation fAnnotation;
  std::string fTitle;
  Lib::Array<double> fArray;
};

}

#endif
