#ifndef BatchLab_FitData_h
#define BatchLab_FitData_h

// Inheritance :
#include <AIDA/IFitData.h>

#include <AIDA/IConstants.h> //For AIDA_VERSION

#include <BatchLab/Core/RangeSet.h>

namespace BatchLab {

class FitData : public virtual AIDA::IFitData {
public: 
  virtual bool create1DConnection(const AIDA::IHistogram1D&);
  virtual bool create1DConnection(const AIDA::ICloud1D&);
  virtual bool create1DConnection(const AIDA::IProfile1D&);
  virtual bool create1DConnection(const AIDA::IDataPointSet&,int,int);

  virtual bool create1DConnection(const std::vector<double>& xs,
                                  const std::vector<double>& ys,
                                  const std::vector<double>& corrMatrix); //AIDA-v3r3p0


  virtual bool create2DConnection(const AIDA::IHistogram2D&);
  virtual bool create2DConnection(const AIDA::IHistogram2D&,int,int);
  virtual bool create2DConnection(const AIDA::ICloud2D&);
  virtual bool create2DConnection(const AIDA::ICloud2D&,int,int);
  virtual bool create2DConnection(const AIDA::IProfile2D&);
  virtual bool create2DConnection(const AIDA::IProfile2D&,int,int);
  virtual bool create2DConnection(const AIDA::IDataPointSet&,int,int,int);

  virtual bool create3DConnection(const AIDA::IHistogram3D&);
  virtual bool create3DConnection(const AIDA::IHistogram3D&,int,int,int);
  virtual bool create3DConnection(const AIDA::ICloud3D&);
  virtual bool create3DConnection(const AIDA::ICloud3D&,int,int,int);
  virtual bool create3DConnection(const AIDA::IDataPointSet&,int,int,int,int);

  virtual bool createConnection(const AIDA::ITuple&,std::vector<AIDA::IEvaluator*>&);

#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual bool createConnection(const AIDA::ITuple&,std::vector<std::string>);
  virtual bool createConnection(const AIDA::IDataPointSet&,std::vector<int>,int);
  virtual AIDA::IRangeSet& range(int) const;
#else
  virtual bool createConnection(const AIDA::ITuple&,const std::vector<std::string>&);
  virtual bool createConnection(const AIDA::IDataPointSet&,const std::vector<int>&,int);
  virtual AIDA::IRangeSet& range(int);
#endif

  virtual void reset();
  virtual int dimension() const;
  virtual std::string dataDescription() const;
public: 
  FitData();
  virtual ~FitData();
private:
  RangeSet fRangeSet;
};

}

#endif
