/*
  OnX is started through the G4Lab::UIOnX (which inherits G4UISession).
 */

// Geant4 :
#include <G4RunManager.hh>
#include <G4UserSteppingAction.hh>

// G4Lab :
#include <G4Lab/UIOnX.h>
#include <G4Lab/TrackingAction.h>

// G4LabTemplateOne :
#include <G4LabTemplateOne/Detector.h>
#include <G4LabTemplateOne/Primary.h>
#include <G4LabTemplateOne/PhysicsList.h>
#include <G4LabTemplateOne/RunAction.h>
#include <G4LabTemplateOne/EventAction.h>
#include <G4LabTemplateOne/SteppingAction.h>

//////////////////////////////////////////////////////////////////////////////
int main(
 int aArgc
,char** aArgv
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  G4RunManager* runManager = new G4RunManager;

  runManager->SetUserInitialization(new G4LabTemplateOne::Detector);
  runManager->SetUserInitialization(new G4LabTemplateOne::PhysicsList); 
  runManager->SetUserAction(new G4LabTemplateOne::PrimaryGeneratorAction);
  runManager->SetUserAction(new G4LabTemplateOne::EventAction);

  // The G4Lab::TrackingAction permits to use the G4Lab::Trajectory
  // that permits to have G4Track infos on picking.
  runManager->SetUserAction(new G4Lab::TrackingAction);

  // Need to pass the G4RunManager at UIOnX creation
  // so that the G4Lab::Manager get it properly.
  std::string gui = 
    "$G4LABTEMPLATEONEROOT/scripts/OnX/G4LabTemplateOne.onx";
  G4Lab::UIOnX* session = new G4Lab::UIOnX(*runManager,gui,aArgc,aArgv);

  runManager->SetUserAction
    (new G4LabTemplateOne::RunAction(session->printer()));

  AIDA::IAnalysisFactory* aida = session->findAIDA();
  runManager->SetUserAction(new G4LabTemplateOne::SteppingAction(aida));

  // Main UI loop :
  // GUI is constructed here.
  // Create callbacks are executed here.
  session->SessionStart();

  delete session;

  delete runManager;

  return 0;
}

