#ifndef OnX_Helpers_OnX_h
#define OnX_Helpers_OnX_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//  It relies on interfaces and basic libraries (Lib, stdcxx) only.
//

// This file contains no Inventor related helpers.
// See OnX/Helpers/Inventor.h for Inventor related ones.

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/Data/IProcessor.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/IScriptManager.h>
#include <Slash/UI/IWidget.h>

// Lib :
#include <Lib/Manager.h>
#include <Lib/fmanip.h>
#include <Lib/smanip.h>
#include <Lib/Cast.h>
#include <Lib/Out.h>
#include <Lib/sout.h>

//////////////////////////////////////////////////////////////////////////////
/// lib //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline std::string lib_torgb(
 const std::string& aColorName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string value;
  Lib::smanip::torgbs(aColorName,value);
  return value;
}
//////////////////////////////////////////////////////////////////////////////
/// sys //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void sys_copy(
 const std::string& aFrom
,const std::string& aTo
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string from = Lib::fmanip::name(aFrom);
  std::string to = Lib::fmanip::name(aTo);
  Lib::fmanip::copy(from,to);
}
//////////////////////////////////////////////////////////////////////////////
inline void sys_remove(
 const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::fmanip::remove(Lib::fmanip::name(aFile));
}
//////////////////////////////////////////////////////////////////////////////
inline void sys_rm(
 const std::string& aWhat
) 
//////////////////////////////////////////////////////////////////////////////
// Handle wildcards.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::fmanip::rm(aWhat);
}
//////////////////////////////////////////////////////////////////////////////
/// session //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void session_setColor(
 Slash::Core::ISession& aSession
,const std::string& aColor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aSession.setParameter("modeling.color",aColor);
}
//////////////////////////////////////////////////////////////////////////////
inline void session_modeling_none(
 Slash::Core::ISession& aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Must match list of SbStyle::setFromString.
  aSession.setParameter("modeling.modeling","none");
  aSession.setParameter("modeling.color","none");
  aSession.setParameter("modeling.highlightColor","none");
  aSession.setParameter("modeling.lineStyle","none");
  aSession.setParameter("modeling.linePattern","none");
  aSession.setParameter("modeling.markerStyle","none");
  aSession.setParameter("modeling.areaStyle","none");
  aSession.setParameter("modeling.cut","none");
  aSession.setParameter("modeling.painting","none");
  aSession.setParameter("modeling.projection","none");
  aSession.setParameter("modeling.fontName","none");
  aSession.setParameter("modeling.lineWidth","none");
  aSession.setParameter("modeling.markerSize","none");
  aSession.setParameter("modeling.fontSize","none");
  aSession.setParameter("modeling.transparency","none");
  aSession.setParameter("modeling.multiNodeLimit","none");
  aSession.setParameter("modeling.hatching","none");
  aSession.setParameter("modeling.angle","none");
  aSession.setParameter("modeling.offset","none");
  aSession.setParameter("modeling.stripWidth","none");
  aSession.setParameter("modeling.spacing","none");
  aSession.setParameter("modeling.visible","none");
  aSession.setParameter("modeling.barOffset","none");
  aSession.setParameter("modeling.barWidth","none");

  aSession.setParameter("modeling.localSetup","none");
  // Backward compatibility :
  aSession.setParameter("modeling.type","none");
}
//////////////////////////////////////////////////////////////////////////////
inline bool session_setParameter(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,int aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%d",aValue);
  return aSession.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
inline bool session_setParameter(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%g",aValue);
  return aSession.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
inline bool session_setParameter(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return aSession.setParameter(aWhat,std::string(aValue?"true":"false"));
}
//////////////////////////////////////////////////////////////////////////////
inline bool session_parameterValue(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,int& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aSession.parameterValue(aWhat,s)) return false;
  int value;
  if(!Lib::smanip::toint(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline bool session_parameterValue(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aSession.parameterValue(aWhat,s)) return false;
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline bool session_parameterValue(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s;
  if(!aSession.parameterValue(aWhat,s)) return false;
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline std::string session_interpreterFromSuffix(
 Slash::Core::ISession& aSession
,const std::string& aSuffix
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IScriptManager* sm = 
    Lib_findManager(aSession,"ScriptManager",Slash::UI::IScriptManager);
  if(!sm) return "";
  Slash::UI::IInterpreter* interp = sm->findInterpreterFromSuffix(aSuffix);
  if(!interp) return "";
  return interp->name();
}
//////////////////////////////////////////////////////////////////////////////
/// ui ///////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void ui_echo_file(
 Slash::UI::IUI& aUI
,const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> text;
  if(!Lib::fmanip::read(aFile,text)) return;
  Slash::Core::ISession& session = aUI.session();
  unsigned int textn = text.size();
  for(unsigned int index=0;index<textn;index++) {
    session.printer().write(text[index]);
    session.printer().write("\n");
  }
}
//////////////////////////////////////////////////////////////////////////////
inline void ui_help_text(
 Slash::UI::IUI& aUI
,const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
// aString : text to output in the help dialog (or terminal).
//           It is a list of strings separated by \n.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  std::string svalue;
  if(!session.parameterValue("session.helpOnDialog",svalue)) return;
  if(svalue=="yes") {
    // Output on dialog :
    aUI.load("main",Lib::fmanip::name("$ONXROOT/scripts/OnX/HelpDialog.onx"));
    aUI.setParameter("OnX_HelpDialog_text.value",aString);
    aUI.show("OnX_HelpDialog_dialog");
  } else {
    std::vector<std::string> text;
    Lib::smanip::words(aString,"\n",text,true);
    // Output on console :
    unsigned int textn = text.size();
    for(unsigned int index=0;index<textn;index++) {
      session.printer().write(text[index]);
      session.printer().write("\n");
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
inline void ui_help_file(
 Slash::UI::IUI& aUI
,const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
// aArgs[0] : file name.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> text;
  if(!Lib::fmanip::read(aFile,text)) return;
  ui_help_text(aUI,Lib::smanip::tostring(text,"\n"));
}
//////////////////////////////////////////////////////////////////////////////
inline void ui_execute_file(
 Slash::UI::IUI& aUI
,const std::string& aInterp
,const std::string& aFile
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IScriptManager* sm = 
    Lib_findManager(aUI.session(),"ScriptManager",Slash::UI::IScriptManager);
  if(!sm) return;
  Slash::UI::IInterpreter* interp = sm->findInterpreter(aInterp);
  if(!interp) return;
  std::string scmd = interp->execFileCommand();
  Lib::smanip::replace(scmd,"onx_file",Lib::fmanip::name(aFile));
  sm->record(aInterp,scmd);
  interp->executeScript(scmd);
}
//////////////////////////////////////////////////////////////////////////////
inline std::string ui_interpreter_suffix(
 Slash::UI::IUI& aUI
,const std::string& aInterp
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IScriptManager* sm = 
    Lib_findManager(aUI.session(),"ScriptManager",Slash::UI::IScriptManager);
  if(!sm) return "";
  Slash::UI::IInterpreter* interp = sm->findInterpreter(aInterp);
  if(!interp) return "";
  return interp->suffix();
}
//////////////////////////////////////////////////////////////////////////////
inline bool ui_setParameter(
 Slash::UI::IUI& aUI
,const std::string& aWhat
,int aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%d",aValue);
  return aUI.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
inline bool ui_setParameter(
 Slash::UI::IUI& aUI
,const std::string& aWhat
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%g",aValue);
  return aUI.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
inline bool ui_setParameter(
 Slash::UI::IUI& aUI
,const std::string& aWhat
,bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return aUI.setParameter(aWhat,std::string(aValue?"true":"false"));
}
//////////////////////////////////////////////////////////////////////////////
inline bool ui_parameterValue(
 Slash::UI::IUI& aUI
,const std::string& aWhat
,int& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) return false;
  int value;
  if(!Lib::smanip::toint(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline bool ui_parameterValue(
 Slash::UI::IUI& aUI
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) return false;
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline bool ui_parameterValue(
 Slash::UI::IUI& aUI
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) return false;
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline double ui_cbk_double(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value;
  Lib::smanip::todouble(aUI.callbackValue(),value);
  return value;
}
//////////////////////////////////////////////////////////////////////////////
inline int ui_cbk_int(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int value;
  Lib::smanip::toint(aUI.callbackValue(),value);
  return value;
}
//////////////////////////////////////////////////////////////////////////////
inline bool ui_cbk_bool(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool value;
  Lib::smanip::tobool(aUI.callbackValue(),value);
  return value;
}
//////////////////////////////////////////////////////////////////////////////
inline std::string ui_cbk(
 Slash::UI::IUI& aUI
,const std::string& aArg
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArg=="ui_callbackValue") return aUI.callbackValue();
  else return aArg;
}
//////////////////////////////////////////////////////////////////////////////
inline void ui_add_prompt(
 Slash::UI::IUI& aUI
,const std::string& aArg
,bool aWarn = true
) 
//////////////////////////////////////////////////////////////////////////////
// Add a prompt in the consolePrompt list.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aWarn) {
    if(!aUI.findWidget("consolePrompt")) return;
  }
  std::string value;
  if(!aUI.parameterValue("consolePrompt.items",value)) return;
  std::vector<std::string> items;
  Lib::smanip::lines(value,items);
  items.push_back(aArg);
  Lib::smanip::unique(items);
  aUI.setParameter("consolePrompt.items",Lib::smanip::tostring(items,"\n"));
}
//////////////////////////////////////////////////////////////////////////////
inline void ui_show_dialog(
 Slash::UI::IUI& aUI
,const std::string& aFile
,const std::string& aDialog
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.load("main",Lib::fmanip::name(aFile));
  aUI.show(aDialog);
}
//////////////////////////////////////////////////////////////////////////////
/// data /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void data_collect(
 Slash::Data::IProcessor& aAccessorManager
,const std::string& aWhat
,const std::string& aCuts = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aCuts);
  aAccessorManager.execute("collect",ss);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_visualize(
 Slash::Data::IProcessor& aAccessorManager
,const std::string& aStyle = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> ss;
  ss.push_back("style="+aStyle);
  aAccessorManager.execute("visualize",ss);
}
//////////////////////////////////////////////////////////////////////////////
inline int data_number(
 Slash::Data::IProcessor& aAccessorManager
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = 
    aAccessorManager.execute("number",std::vector<std::string>());
  int number;
  if(!Lib::smanip::toint(s,number)) return 0;
  return number;  
}
//////////////////////////////////////////////////////////////////////////////
inline std::vector<std::string> data_values(
 Slash::Data::IProcessor& aAccessorManager
,const std::string& aWhat
) 
//////////////////////////////////////////////////////////////////////////////
// returned string is a \n separated list.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  return Lib::smanip::words(aAccessorManager.execute("values",ss),"\n");
}
//////////////////////////////////////////////////////////////////////////////
inline void data_dump(
 Slash::Data::IProcessor& aAccessorManager
,const std::string& aMode = "table"
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> ss;
  ss.push_back(aMode);
  aAccessorManager.execute("dump",ss);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_filter(
 Slash::Data::IProcessor& aAccessorManager
,const std::string& aWhat
,const std::string& aFilter = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aFilter);
  aAccessorManager.execute("filter",ss);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_set(
 Slash::Data::IProcessor& aAccessorManager
,const std::string& aWhat
,const std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aValue);
  aAccessorManager.execute("set",ss);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_destroy(
 Slash::Data::IProcessor& aAccessorManager
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aAccessorManager.execute("destroy",std::vector<std::string>());
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void data_collect(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,const std::string& aCuts = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return;
  data_collect(*accessorManager,aWhat,aCuts);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_visualize(
 Slash::Core::ISession& aSession
,const std::string& aStyle = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return;
  data_visualize(*accessorManager,aStyle);
}
//////////////////////////////////////////////////////////////////////////////
inline int data_number(
 Slash::Core::ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return 0;
  return data_number(*accessorManager);
}
//////////////////////////////////////////////////////////////////////////////
inline std::vector<std::string> data_values(
 Slash::Core::ISession& aSession
,const std::string& aWhat
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return std::vector<std::string>();
  return data_values(*accessorManager,aWhat);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_dump(
 Slash::Core::ISession& aSession
,const std::string& aMode = "table"
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return;
  data_dump(*accessorManager,aMode);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_filter(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,const std::string& aFilter = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return;
  data_filter(*accessorManager,aWhat,aFilter);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_set(
 Slash::Core::ISession& aSession
,const std::string& aWhat
,const std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return;
  data_set(*accessorManager,aWhat,aValue);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_destroy(
 Slash::Core::ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return;
  data_destroy(*accessorManager);
}
//////////////////////////////////////////////////////////////////////////////
inline void data_toggle(
 Slash::Core::ISession& aSession
,const std::string& aName
,const std::string& aColor
,const std::string& aStatus
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aStatus=="true") {
    session_setColor(aSession,aColor);
    data_collect(aSession,aName);
    data_number(aSession);
    data_visualize(aSession);
  } else {
    std::string filter = "name==\""+aName+"/*\"";
    data_collect(aSession,"SceneGraph",filter);
    data_destroy(aSession);
  }
}


//////////////////////////////////////////////////////////////////////////////
/// tree /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void tree_set_with_strings(
 Slash::UI::IUI& aUI
,const std::string& aParameter
,const std::vector<std::string>& aList
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sout = "<tree>";
  for(unsigned int index=0;index<aList.size();index++) {
    sout += "<treeItem><label>"+aList[index]+"</label></treeItem>";
  }
  sout += "</tree>";
  aUI.setParameter(aParameter,sout);
}
//////////////////////////////////////////////////////////////////////////////
inline void tree_set(
 Slash::UI::IUI& aUI
,const std::string& aParameter
,const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
// Example : tree_set(aUI,"mainTree.items","aa\nbb")
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> items;
  Lib::smanip::lines(aString,items);
  tree_set_with_strings(aUI,aParameter,items);
}
//////////////////////////////////////////////////////////////////////////////
inline void tree_set_with_file(
 Slash::UI::IUI& aUI
,const std::string& aParameter
,const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> lines;
  if(!Lib::fmanip::read(aFile,lines)) {
    aUI.echo("Can't open \""+aFile+"\" file.");
  } else {
    aUI.setParameter(aParameter,Lib::smanip::tostring(lines,""));
  }
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
inline void editExec_setFile(
 Slash::UI::IUI& aUI
,const std::string& aFile
,bool aMap = true
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> text;
  if(!Lib::fmanip::read(aFile,text)) return;
  if(aMap) {
    ui_show_dialog(aUI,
                   "$ONXROOT/scripts/OnX/EditExec.onx",
                   "OnX_EditExec_main");
  }
  aUI.setParameter("OnX_EditExec_text.value",Lib::smanip::tostring(text,"\n"));
  // Set interpreter :
  std::string suffix = Lib::smanip::suffix(aFile);
  std::string interp = session_interpreterFromSuffix(aUI.session(),suffix);
  if(interp=="") interp = "DLD";
  aUI.setParameter("OnX_EditExec_interpreter.default",interp);
}

#endif
