#ifndef OnX_QtUI_h
#define OnX_QtUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#include <vector>
#include <list>

#include <qglobal.h> //For QT_VERSION.

class QApplication;
class QWidget;
class QPopupMenu;
class QMessageBox;
class QObject;
class QEventLoop;

class QtCallback;
class OnX_QMessageWidget;
class QListViewItem;

#if QT_VERSION < 0x040000
class QPopupMenu;
#else
#ifdef QT3_SUPPORT
class Q3PopupMenu;
#define QPopupMenu Q3PopupMenu
#else
#error "OnX on Qt4 without -DQT3_SUPPORT not yet supported."
#endif
#endif

namespace OnX {class TermCyclic;}

namespace OnX {

class QtUI : public BaseUI {
public: //Slash::UI::IUI
  virtual void* cast(const std::string&) const;
  virtual bool isValid() const;
  virtual void put(const std::string&);
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();

  virtual bool setCallback(const std::string&,
                           const std::string&,
                           const std::string&,
                           const std::string&,
                           int = 0);
  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool position(const std::string&,int&,int&);

  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string& widget,
                     const std::string& file,
                     const std::string& format,
                     const std::string& options = "");
  virtual Slash::UI::ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Slash::Core::IMessage*);
  virtual bool enableTerminal(const std::string&,const std::string&);
  virtual bool disableTerminal();
  virtual std::string ask(const std::string&);
protected: //OnX::BaseUI
  virtual int nativeSteer();
  virtual bool nativeSize(void*,unsigned int&,unsigned int&);
  virtual bool createNativeWidget(Lib::ItemML&,
                            const std::string&,
                            const std::string&,
                            Lib::ItemML*);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
public:
  QtUI(Slash::Core::ISession&,const std::string&,const std::vector<std::string>&);
  virtual ~QtUI();
  QApplication* getQApplication() const;
private:
  bool createTree(QWidget*,Lib::ItemML*);
  //bool setExpand(QWidget*,const std::string);
  //unsigned int getQtDecor(const std::string&,const std::string&);
  int getCompound(const std::string&);
  std::string getType(const std::string&);
  bool destroyOneWidget(Lib::ItemML&);
  void addCallbacks(Lib::ItemML&,QWidget*,
                    const std::string&,
                    const std::string&,
                    const std::string& = "");
  void addCallbacks(Lib::ItemML&,QPopupMenu*,int aActualID,const std::string&);
  std::string getCallbackName(const QObject&,const std::string&);
  static std::string getXMLTree(QListViewItem*);
protected:
  QApplication* fQApplication;
private:
  bool fQApplication_owner;
  bool fFirstWindow;
  // Console panel :
  QWidget* fConsoleWindow;
  QWidget* fConsoleText;
  QMessageBox* fWarningDialog;
  int fCounter;
  int fMenuItemCounter;
  std::vector<QtCallback*> fQtCallbacks;
  OnX_QMessageWidget* fMessageWidget;
  std::list<QWidget*> fWidgets;
  std::list<QEventLoop*> fEventLoopStack;
  TermCyclic* fTermCyclic;
};

}

#if QT_VERSION < 0x040000
#else
#undef QPopupMenu
#endif

#endif

