#ifndef OnX_XtUI_h
#define OnX_XtUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#include <X11/Intrinsic.h>
typedef Widget XtWidget;

#include <Xm/Xm.h>

namespace OnX {class TermCyclic;}

namespace OnX {

class XtUI : public BaseUI {
public: //Slash::UI::IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();
  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool position(const std::string&,int&,int&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string& widget,
                     const std::string& file,
                     const std::string& format,
                     const std::string& options = "");
  virtual Slash::UI::ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Slash::Core::IMessage*);
  virtual bool enableTerminal(const std::string&,const std::string&);
  virtual bool disableTerminal();
protected: //OnX::BaseUI
  virtual int nativeSteer();
  virtual bool nativeSize(void*,unsigned int&,unsigned int&);
  virtual bool createNativeWidget(Lib::ItemML&,
                            const std::string&,
                            const std::string&,
                            Lib::ItemML*);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
  virtual std::string ask(const std::string&);
public:
  /** 
   * @param aSession : a valid Slash::Core::ISession.
   * @param aFile : .onx file.
   * @param aArgs : argc, argv coming from main, needed to initialize Xt.
   */
  XtUI(Slash::Core::ISession& aSession,
       const std::string& aFile,const std::vector<std::string>& aArgs);

  virtual ~XtUI();
  bool createTree(XtWidget,Lib::ItemML*);
  XtAppContext appContext() const;
  XtWidget appWidget() const;
private:
  static void callback(XtWidget,XtPointer,XtPointer);
  static void destroyCallback(XtWidget,XtPointer,XtPointer);
  static void warningCallback(XtWidget,XtPointer,XtPointer);
private:
  //bool setSplitPosition(XtWidget,int,const std::string);
  int getCompound(const std::string&);
  WidgetClass getType(const std::string&);
  std::string getCallbackName(WidgetClass,const std::string&);
  void addCallbacks(Lib::ItemML&,XtWidget,const std::string&,const std::string&);
protected:
  XtWidget fAppWidget;
  XtAppContext fAppContext;
private:
  std::list<XtWidget> fWidgets;
  // Console panel :
  XtWidget fConsoleWindow;
  XtWidget fConsoleText;
  XtWidget fWarningDialog;
  TermCyclic* fTermCyclic;
};

}

#endif

